#ifndef LINUX_NBD_H
#define LINUX_NBD_H

#define NBD_VERSION     1

#define NBD_SET_SOCK	_IO( 0xab, 0 )
#define NBD_SET_BLKSIZE	_IO( 0xab, 1 )
#define NBD_SET_SIZE	_IO( 0xab, 2 )
#define NBD_DO_IT	_IO( 0xab, 3 )
#define NBD_CLEAR_SOCK	_IO( 0xab, 4 )
#define NBD_CLEAR_QUE	_IO( 0xab, 5 )
#define NBD_PRINT_DEBUG	_IO( 0xab, 6 )
#define NBD_SET_SIZE_BLOCKS	_IO( 0xab, 7 )
#define NBD_SET_SERVER_FLAGS	_IO( 0xab, 8 )
#define NBD_SET_SERVER_VERSION	_IO( 0xab, 9 )
#define NBD_SET_CLIENT_FLAGS	_IO( 0xab, 10 )
#define NBD_SET_SERVER_TICKLE   _IO( 0xab, 11 )

#ifdef MAJOR_NR

#include <linux/locks.h>
#include <asm/semaphore.h>

#define LOCAL_END_REQUEST

#include <linux/blk.h>

#ifdef PARANOIA
extern int requests_in;
extern int requests_out;
#endif

static void 
nbd_end_request(struct request *req)
{
#ifdef PARANOIA
	requests_out++;
#endif
	if (end_that_request_first( req, !req->errors, "nbd" ))
		return;
	end_that_request_last( req );
}

#define MAX_NBD 128

struct nbd_device {
	int refcnt;	
	int flags;
	int harderror;		/* Code of hard error			*/
#define NBD_READ_ONLY 0x0001
#define NBD_WRITE_NOCHK 0x0002
#define NBD_INITIALISED 0x0004
	struct socket * sock;
	struct file * file; 		/* If == NULL, device is not ready, yet	*/
	int magic;			/* FIXME: not if debugging is off	*/
	struct request *head;	/* Requests are added here...			*/
	struct request *tail;
	struct semaphore queue_lock;
        u16 s_version, s_flags;         /* server version and flags */
	u16 c_flags, replysize;         /* client flags, reply size */
	unsigned long timeout;          /* timeout to use */
	void *istreamp, *ostreamp;      /* for compression */
	void *zbuf;                     /* scratch compression buffer */
	int zlen;                       /* not really necessary, but useful */
};
#endif

/* This now IS in some kind of include file...	*/

/* nbd request types. the nbd protocol didn't use a different namespace 
 * from linux/fs.h, so we have some backwards compatibility gunk to 
 * deal with. */
#define NBD_OP_READ      0
#define NBD_OP_WRITE     1
#define NBD_OP_READA     2   /* not used by nbd */
#define NBD_OP_WRITEA    3   /* not used by nbd */

#define NBD_OP_CONFIG    100 /* client-supported versions/options */
#define NBD_OP_READZ     101 /* compressed read request */
#define NBD_OP_WRITEZ    102 /* compressed write request */
#define NBD_OP_DONE      103 /* done! */
#define NBD_OP_TICKLE    104 /* tickle timeouts */
#define NBD_OP_FLUSH     105 /* flush any outstanding requests */

/* flags */
#define NBD_FLAG_GZIP    0x0001

/* These are send over network in request/reply magic field */

#define NBD_REQUEST_MAGIC 0x25609513
#define NBD_REPLY_MAGIC 0x67446698
/* Do *not* use magics: 0x12560953 0x96744668. */

/*
 * This is packet used for communication between client and
 * server. All data are in network byte order.
 */
#define NBD_REQUEST_SIZE  28
struct nbd_request {
	u32 magic;
	u32 type;	/* == READ || == WRITE 	*/
	char handle[8];
	u64 from;
	u32 len;
}
#ifdef __GNUC__
	__attribute__ ((packed))
#endif
;

#define NBD_REPLY_SIZE_V0 16
#define NBD_REPLY_SIZE    28
struct nbd_reply {
	u32 magic;
	u32 error;		/* 0 = ok, else error	*/
	char handle[8];		/* handle you got from request	*/
        u32 type;		/* yes, we might want to set the type */
        u32 filler;             /* filler */
        u32 len;                /* len */
};

#define NBD_DATA_MAX 10240
#endif
